#include "connect_dlg.h"
#include "dc_parse.h"
#include "hub_lv_item.h"
#include "dc_settings.h"
#include "util.h"

#include <qlistview.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtabwidget.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlistbox.h>
#include <qinputdialog.h>
#include <qpopupmenu.h>
#include <qlineedit.h>

#include <procbuf.h>

connect_dlg::connect_dlg(dc_hub *h, 
	QWidget *p /*= 0*/, const char *name /*= 0*/, bool modal /*= false*/)
	: QDialog(p, name, modal)
{	
	hubs = h;
	
	tab = new QTabWidget(this,"connection tab");
	
	QVBox *tab1 = new QVBox;
	QHBox *tab2 = new QHBox;
	QHBox *tab3 = new QHBox;

	// Create tab 1.
	
	// list view of hubs in the first row
	// three buttons side by side along the bottom
	gl_layout = new QGridLayout(this, 2, 3);
	
	lv_hubs = new QListView(tab1, "connect_dlg_hubs");
	lv_hubs->addColumn("Hub Name");
	lv_hubs->addColumn("Hub IP");
	lv_hubs->addColumn("Comment");
	lv_hubs->addColumn("Users");

	lv_hubs->setColumnWidthMode(1, QListView::Manual);
	lv_hubs->setColumnWidthMode(2, QListView::Manual);
	lv_hubs->setColumnWidthMode(3, QListView::Manual);

	lv_hubs->setColumnWidth(1, 150);
	lv_hubs->setColumnWidth(2, 300);
	lv_hubs->setColumnWidth(3, 50);

	hubs->load_list_from_disk();
	hubs->load_running_hubs();
	
	public_hubs = hubs->get_hubs();
	add_hubs_to_list(public_lv_items, lv_hubs, public_hubs);

	menu = new QPopupMenu(lv_hubs,"connection menu");
	menu->insertItem("Add to favourites");
	connect(menu,SIGNAL(activated(int)),SLOT(menu_clicked(int)));
	connect(lv_hubs,SIGNAL(rightButtonClicked(QListViewItem*,const QPoint&,int)),
		SLOT(show_menu(QListViewItem*,const QPoint&,int)));

	tab->addTab(tab1,"&Public");

	// Create tab 2... this is one mess, need to shape up the layout later.
	QVBox *vbox = new QVBox(tab2);
	QPushButton *newBtn = new QPushButton("New",vbox);
	QPushButton *editBtn = new QPushButton("Edit",vbox);
	QPushButton *removeBtn = new QPushButton("Remove",vbox);
	vbox->setMaximumWidth(100);
	favList = new QListView(tab2,"favourites list");
	favList->addColumn("Name");	
	favList->addColumn("Address");
	vbox->layout()->addItem(new QSpacerItem(10,300));
	favList->setMaximumWidth(400);

	for(int i=0;i < settings.num_favourites();i++) 
	{
		QString name,address;
		settings.get_favourite(i,name,address);
		//favList->insertItem(name);
		QListViewItem *item = new QListViewItem(favList,name,address);
	}
	
	tab->addTab(tab2,"&Favourites");
 	tab2->setStretchFactor(favList,20);
	tab2->setStretchFactor(vbox,1);
	QSpacerItem *spacer = new QSpacerItem(300,10,QSizePolicy::Expanding,QSizePolicy::Expanding);
 	spacer->setAlignment(Qt::AlignRight);
 	tab2->layout()->addItem(spacer);

	connect(newBtn, SIGNAL(clicked()), this, SLOT(addFavourite()));
	connect(editBtn, SIGNAL(clicked()), this, SLOT(editFavourite()));
	connect(removeBtn, SIGNAL(clicked()), this, SLOT(removeFavourite()));
	
	// Tab 3
	QVBox *running_vbox = new QVBox(tab3);
	run_lv = new QListView(tab3, "running list");
	run_lv->addColumn("Hub Name");
	run_lv->addColumn("Hub IP");
	run_lv->addColumn("Comment");
	run_lv->addColumn("Users");
	running_vbox->layout()->addItem(new QSpacerItem(10, 300));
	
	running_hubs = hubs->get_running_hubs();
	add_hubs_to_list(running_lv_items, run_lv, running_hubs);
	
	tab->addTab(tab3, "Running");
	tab3->layout()->setSpacing(0);

	gl_layout->addMultiCellWidget(tab, 0, 0, 0, 2);
	
	// Add the rest - non-tab stuff
	pb_ok = new QPushButton("&Ok", this);
	pb_cancel = new QPushButton("&Cancel", this);
	pb_get_hubs = new QPushButton("&Refresh Listing", this);
	gl_layout->addWidget(pb_ok, 1, 1);
	gl_layout->addWidget(pb_cancel, 1, 2);
	gl_layout->addWidget(pb_get_hubs, 1, 0);

	// size dialog
	setMaximumWidth(1280);
	
	connect(pb_ok, SIGNAL(clicked()), SLOT(accept()));
	connect(pb_cancel, SIGNAL(clicked()), SLOT(reject()));
	connect(pb_get_hubs, SIGNAL(clicked()), SLOT(refresh_listing()));
}

connect_dlg::~connect_dlg()
{
	clear_list(public_lv_items);
	clear_list(running_lv_items);
}

void connect_dlg::add_hubs_to_list(vector<hub_lv_item*> &l, QListView *lv, const vector<hub_info*> &h)
{
	for (int i = 0; i < h.size(); i++)
		l.push_back(new hub_lv_item(lv, h[i]->name, h[i]->ip, h[i]->comment, h[i]->users));
}

void connect_dlg::clear_list(vector<hub_lv_item*> &l)
{
	while (l.size() > 0)
	{
		hub_lv_item* h = l.back();
		l.pop_back();
		delete h;
	}
}

const hub_info connect_dlg::selected_hub()
{
	if(tab->currentPageIndex() == 0) 
	{
		if (!lv_hubs->selectedItem())
			return hub_info("", "", "", "");

		return hubs->get_public_hub(lv_hubs->selectedItem()->text(HUBIP));
	}
	if(tab->currentPageIndex() == 1) 
	{
		if(favList->currentItem() == NULL) 
			return hub_info("", "", "", "");
		return settings.get_favourite(favList->selectedItem()->text(0));
	}
	if(tab->currentPageIndex() == 2)
	{
		if (!run_lv->currentItem())
			return hub_info("", "", "", "");
		return hubs->get_running_hub(run_lv->selectedItem()->text(HUBIP));
	}
}

bool connect_dlg::selected_running()
{
	return tab->currentPageIndex() == 2;
}

void connect_dlg::refresh_listing()
{
	clear_list(public_lv_items);
	hubs->load_list_from_web();
	vector<hub_info*> h = hubs->get_hubs();
	add_hubs_to_list(public_lv_items, lv_hubs, h);
}

void connect_dlg::addFavourite()
{
	QString name = QInputDialog::getText("Enter an alias","Enter an alias:");
	QString address = QInputDialog::getText("Enter address","Enter a hostname or IP:");

	QListViewItem *item = new QListViewItem(favList,name,address);
	settings.add_favourite(name,address);
	
}

void connect_dlg::editFavourite()
{
	QString current_name = favList->selectedItem()->text(0);
	QString current_ip = settings.get_favourite(current_name).ip;
	QString name = QInputDialog::getText("Enter an alias","Enter an alias:",QLineEdit::Normal,
						current_name);
	QString address = QInputDialog::getText("Enter address","Enter a hostname or IP:",
						QLineEdit::Normal,current_ip);
	settings.add_favourite(name,address);

#ifdef _QT_3
	favList->findItem(current_name,0)->setText(0,name);
	favList->findItem(name,0)->setText(1,address);
#else
#warning QListViewItem::findItem is qt3 only
#endif
}

void connect_dlg::removeFavourite()
{
	QListViewItem *item = favList->currentItem();
	settings.del_favourite( item->text(0) );
	favList->takeItem( item );
	delete item;
}

void connect_dlg::menu_clicked(int i)
{
	if(last_clicked)
	{
		QListViewItem *item = new QListViewItem(favList,
							last_clicked->text(0),
							last_clicked->text(1));	
		settings.add_favourite(last_clicked->text(0),
					last_clicked->text(1));
	}
}

void connect_dlg::show_menu(QListViewItem* item,const QPoint &p, int)
{
	menu->popup(p);
	last_clicked = item;
}

