/* This file is part of dc_qt.

    dc_qt is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    dc_qt is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dc_qt; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "dc_hub.h"
#include "dc_parse.h"
#include "dc_settings.h"

#include <procbuf.h>

#include <qdir.h>

const char *HUBLIST = "hublist";

dc_hub::dc_hub()
{
}

void dc_hub::load_list_from_web()
{
	load_list_from_process("hublist | tee " + settings.get_home_dir() + "/.dc_qt/hublist.cache");
}

void dc_hub::load_list_from_disk()
{
	load_list_from_process("cat " + settings.get_home_dir() + "/.dc_qt/hublist.cache");
}

void dc_hub::load_list_from_process(const QString& process)
{
	clear_list(hubs);
	
	procbuf *proc = new procbuf(process, ios::in);
	istream in(proc);
	
	for (int i = 0;!in.eof(); i++)
	{
		char buf[10000];
		
		in.getline(buf, 10000);

		dc_parse p(buf, '|');

		if (p.num_strs() >= 2)
		{
			hub_info *h = new hub_info(p.get_str(HUBNAME), p.get_str(HUBIP), 
					p.get_str(COMMENT), p.get_str(NUMUSERS));

			hubs.push_back(h);
		}
	}

	delete proc;
	proc = 0;
}

dc_hub::~dc_hub()
{
	clear_list(hubs);
	clear_list(running);
}

void dc_hub::clear_list(vector<hub_info*> &l)
{
	while (!l.empty())
	{
		hub_info *h = l.back();
		l.pop_back();
		delete h;
	}
}

hub_info dc_hub::get_hub_info(const QString &ip)
{
	for (int i = 0; i < hubs.size(); i++)
	{
		if (hubs[i]->ip == ip)
			return hub_info(hubs[i]->name, hubs[i]->ip, hubs[i]->comment, hubs[i]->users);
	}

	return hub_info("", "", "", "");
}

void dc_hub::load_running_hubs()
{
	clear_list(running);
	
	const QString dir_name = settings.get_home_dir() + "/.dctc/running";

	QDir dir(dir_name, "*", QDir::Name|QDir::IgnoreCase, QDir::System);

	for (int i = 0; i < dir.count(); i++)
	{
		dc_parse p(dir[i], '-');

		QString pid_hex = p.get_str(1);
		QString ip = p.get_str(2);

		hub_info *info = new hub_info(get_hub_info(ip));
		
		if (*info != hub_info("", "", "", ""))
		{
			QString ptemp = pid_hex;
			bool ok;
			info->pid = ptemp.toInt(&ok, 16);
			running.push_back(info);
		}
		else
			delete info;
	}
}

const hub_info dc_hub::get_public_hub(const QString &ip)
{
	for (int i = 0; i < hubs.size(); i++)
	{
		if (hubs[i]->ip == ip)
			return *hubs[i];
	}

	return hub_info("", "", "", "");
}

const hub_info dc_hub::get_running_hub(const QString &ip)
{
	for (int i = 0; i < running.size(); i++)
	{
		if (running[i]->ip == ip)
			return *running[i];
	}
	return hub_info("", "", "", "");
}
