#ifndef DC_SETTINGS_H
#define DC_SETTINGS_H

#include "hub_info.h"

#include <set>
#include <map>
#include <qstring.h>

typedef map<int,QString> Map;
typedef map<QString,QString> StrMap;

class QStringList;

class dc_settings
{
public:
  enum {NICKNAME,DESCRIPTION,CONNECTION,SLOTS,ADDRESS,
	PORT,PASSIVE,LIMIT,OFFSET,DLDIR};

	dc_settings();

	void load_from_file();
	void load_favourites_file();
	void save_to_file();
	void save_favourites_file();
       
	void add_share(const QString &dir);
	void del_share(const QString &dir);
	void clear_shares();

	void add_favourite(const QString &name,const QString &address);
	void del_favourite(const QString &name);

	int num_favourites() const {return favourites.size();}
	bool get_favourite(int i,QString &name,QString &address);
	const hub_info get_favourite(const QString &name);

	QStringList get_connection_types();

	//	QString get_setting(int n) { return settings[n]; }
	QString get_setting(int key);
	void set_setting(int key,const QString &value);

#ifdef _DEBUG
	void print();
#endif

	int num_shares() const {return shares.size();}
	const QString get_share(int i);
	const QString get_home_dir() const { return user_home_dir; }

private:
	int get_setting_key(const QString &s) const;

	QString settings_file;
	QString favourites_file;
	
	set<QString> shares;
	Map settings;
	StrMap favourites;
	QString user_home_dir;
};

extern dc_settings settings;

#endif
