#ifndef _DCTC_HUB_H
#define _DCTC_HUB_H

#ifndef _VERBOSE
#define log(x) ((void)0)
#endif

#include <qobject.h>

class dc_user;
class dctc_connection;

class dctc_hub : public QObject
{
	Q_OBJECT
	
public:
	dctc_hub(QObject *parent = 0, const char* name = 0);
	~dctc_hub();
	
	int hubconnect(const QString &ip, int pid = 0);
	void hubdisconnect();

	void get_uinfo(const QString &user);
	void search_for(const QString &pattern, int type = 1);
	void download_file(const dc_user &from_user, const QString &file);
	void send_chatmsg(const QString &msg);
	void get_ulist() { send_command("/ULIST"); }
	const QString &get_hubip() const { return hubip; }

	// these two are public for debugging only!
#ifdef _DEBUG
public:
#else
private:
#endif
	void send_command(const QString &command);
	QString message;
	
signals:
	void user_added(const QString &, bool = false);
	void user_removed(const QString &);
	void user_info(const QString&, const QString&, const QString&, const QString&);
	void data_to_send(const QString&);
	void logging(const QString &);
	void hub_died();
	void user_flist_file(const QString&, const QString&, const QString &);
	void search_file(const QString&, const QString&, const QString &, const QString &);
	void new_transfer(int, const QString &, const QString &, const QString &);
	void done_transfer(const QString &, const QString &);
	void done_transfer(int);
	void chat_msg(const QString&);
	
private slots:
	void dc_string(const QString&);
	void get_user_list(const QString&);
	void get_file (const QString& user, const QString& file);
	void cancel_transfer(int id);

private:
	void parse_dctc_string(const QString &str);
	void parse_uinfo(const QString &str);
	void parse_file(const QString &str, QString& user, QString &file, QString &size);
	void parse_srest(const QString &str, QString& user, QString &file, QString &size, QString &slot);
	void parse_xferr(const QString &str);
	QString dctc_command_type(const QString &str);
	QString dctc_command_data(const QString &str);

	dctc_connection *connection;
	QString hubip;
};

#endif
