/* This file is part of dc_qt.

    dc_qt is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    dc_qt is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dc_qt; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "file_list_dialog.h"
#include "dc_parse.h"
#include "file_lv_item.h"
#include "file_lv.h"

#include <qlistview.h>
#include <qbutton.h>
#include <qlayout.h>
#include <qpopupmenu.h>

const int GET_FILE = 0;
const int GET_FLIST = 1;

// I'd like to add the ability to grep the files in a file_list_dialog
// I've got a plan, but right now too much doesn't work at all.
// 
// This dialog should be used for search results too.  That'll require
// a little bit of modification.  Inheriting may be better.

list_dialog::list_dialog(QWidget *parent, const char *name)
	: QDialog (parent, name)
{
	resize (600, 800);

	layout = new QVBoxLayout(this);

	menu = new QPopupMenu(this, "file_list_dlg_menu");
	menu->insertItem("Get File", GET_FILE);

	connect (menu, SIGNAL (activated (int)), SLOT (menu_clicked (int)));
}

void list_dialog::show_menu (QListViewItem *i, const QPoint &p, int)
{
	menu->popup (p);
	last_clicked = (directory_lv_item*)i;
}

void list_dialog::clear_files()
{
	file_lv->clear_files();
}

file_list_dialog::file_list_dialog(const QString &u, QWidget* parent, const char *name)
	: list_dialog(parent, name), user(u)
{
	file_lv = new file_dir_lv(this, "dir view list");
	layout->addWidget(file_lv);

	file_lv->addColumn("File");
	file_lv->addColumn("Size");

	file_lv->setColumnWidthMode(0, QListView::Manual);
	file_lv->setColumnWidth(0, 500);

	connect (file_lv, SIGNAL (rightButtonClicked (QListViewItem *, const QPoint &, int)),
			SLOT (show_menu (QListViewItem *, const QPoint &, int)));

	last_clicked = 0;
}

void file_list_dialog::add_file(const QString &full_file_name, const QString &size)
{
	((file_dir_lv*)file_lv)->add_file(full_file_name, size);
}

void file_list_dialog::menu_clicked (int i)
{
	if (i == GET_FILE && last_clicked)
		emit get_file (user, last_clicked->full_name());
}

search_list_dialog::search_list_dialog(const QString &sid, const QString &p, QWidget *parent, 
		const char *name)
	: list_dialog(parent, name), search_id(sid), pattern(p)
{
	file_lv = new search_dir_lv(this, "search view list");
	layout->addWidget(file_lv);

	file_lv->addColumn("File");
	file_lv->addColumn("Size");

	file_lv->setColumnWidthMode(0, QListView::Manual);
	file_lv->setColumnWidth(0, 500);

	connect (file_lv, SIGNAL (rightButtonClicked (QListViewItem *, const QPoint &, int)),
			SLOT (show_menu (QListViewItem *, const QPoint &, int)));

	menu->insertItem("Get File List", GET_FLIST);

	last_clicked = 0;
}

void search_list_dialog::menu_clicked(int i)
{
	if (i == GET_FILE && last_clicked)
	{
//		cerr << "last_clicked->get_root_name(): " << last_clicked->get_root_name() << endl;
//		cerr << "last_clicked->full_name(): " << last_clicked->full_name() << endl;
		emit get_file (last_clicked->get_root_name(), last_clicked->full_name());
	}
	else if (i == GET_FLIST && last_clicked)
	{
		emit get_flist (last_clicked->get_root_name());
	}
}

void search_list_dialog::add_file(const QString &user, const QString &file, const QString &size, 
		const QString &slot)
{
	if (file.contains(pattern, false))
		((search_dir_lv*)file_lv)->add_file(user, file, size, slot);
}

bool operator==(file_list_dialog* f, const QString &s)
{
	return f->get_user() == s;
}

bool operator!=(file_list_dialog* f, const QString &s)
{
	return f->get_user() != s;
}
