#ifndef FILE_LIST_DIALOG_H
#define FILE_LIST_DIALOG_H

#include <qdialog.h>
#include <qstring.h>

class QListView;
class QButton;
class QVBoxLayout;
class QPopupMenu;
class directory_lv;
class directory_lv_item;
class QListViewItem;

class list_dialog : public QDialog
{
	Q_OBJECT

public:
	list_dialog(QWidget* parent = 0, const char *name = 0);
	void clear_files();

protected slots:
	void show_menu(QListViewItem *, const QPoint &, int);
	virtual void menu_clicked(int) = 0;
	
protected:
	QButton *close;
	QVBoxLayout *layout;
	QPopupMenu *menu;
	directory_lv* file_lv;
	directory_lv_item *last_clicked;
};

class file_list_dialog : public list_dialog
{
	Q_OBJECT

public:
	file_list_dialog(const QString &u, QWidget* parent = 0, const char * name = 0);

	void add_file(const QString &file, const QString &size);
	
	QString get_user() { return user; }

signals:
	void get_file(const QString &user, const QString &file);

private slots:
	virtual void menu_clicked (int);

private:
	// can't use the four argument add_file
	void add_file(const QString &, const QString &, const QString &, const QString&) {}

private:
	QString user;
};

class search_list_dialog : public list_dialog
{
	Q_OBJECT
	
public:
	search_list_dialog(const QString &sid, const QString &pattern, QWidget *parent = 0, const char *name = 0);

	virtual void add_file(const QString &user, const QString &file, const QString &size, const QString &slot);

signals:
	void get_file(const QString &, const QString&);
	void get_flist(const QString &);

private slots:
	virtual void menu_clicked(int);

private:
	QString search_id;
	QString pattern;
};

bool operator==(file_list_dialog* f, const QString &s);
bool operator!=(file_list_dialog* f, const QString &s);

#endif
