/* This file is part of dc_qt.

    dc_qt is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    dc_qt is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dc_qt; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "file_lv_item.h"
#include "file_lv.h"
#include "util.h"
#include "pixmap.h"
#include "dc_parse.h"

#include <algorithm>

#include <qpixmap.h>

directory_lv::directory_lv( QWidget *parent, const char *name, bool sdo )
    : QListView( parent, name )//,  oldCurrent( 0 ),
      //dropItem( 0 ), mousePressed( FALSE )
{
    if ( !folderLocked ) 
	{
		folderLocked = new QPixmap( folder_locked );
		folderClosed = new QPixmap( folder_closed_xpm );
		folderOpen = new QPixmap( folder_open_xpm );
		fileNormal = new QPixmap( pix_file );
    }

    connect( this, SIGNAL( doubleClicked( QListViewItem * ) ),
	     this, SLOT( slotFolderSelected( QListViewItem * ) ) );
    connect( this, SIGNAL( returnPressed( QListViewItem * ) ),
	     this, SLOT( slotFolderSelected( QListViewItem * ) ) );

#ifdef _QT_3
	setColumnWidthMode (0, QListView::Manual);
	setColumnWidth(0, 300);
#endif
}

void directory_lv::slotFolderSelected( QListViewItem *i )
{
    if ( !i )
	return;

    directory_lv_item *dir = (directory_lv_item*)i;
    emit folderSelected( dir->full_name() );
}

// I really doubt this works, I have no need for it now
// Should use the dc_file full_path_name anyway
QString directory_lv::fullPath(QListViewItem* item)
{
    QString fullpath = item->text(0);
    while ( (item=item->parent()) ) {
	if ( item->parent() )
	    fullpath = item->text(0) + "/" + fullpath;
	else
	    fullpath = item->text(0) + fullpath;
    }
    return fullpath;
}

void directory_lv::add_file(directory_lv_item *root, const QString &file, const QString &size)
{
	dc_parse p(file, '\\');

	// for each dir in the file name, try to find each directory in the existing
	// tree and if not found, add it
	
	directory_lv_item *cur = root;

	// go up to, but not including the last part of the path, ie stop on the c
	// a/b/c/d since d is not a directory
	for (int i = 0; i < p.num_strs() - 1; i++)
	{
		QString local_dir = p.get_str(i);

		if (cur->get_file(local_dir) == 0)
		{
			directory_lv_item *d = new directory_lv_item(cur, local_dir);
			cur->add_dir (d);
			cur = d;
		}
		else
		{
			cur = (directory_lv_item*)cur->get_file(local_dir);
		}
	}

	// the directories should all be there, now add the file
	if (cur->get_file(p.get_str(p.num_strs() - 1)) == 0)
	{
		file_lv_item * f = new file_lv_item(cur, p.get_str(p.num_strs() - 1), size);
		cur->add_file (f);
	}
}

// could be useful... later
/*void directory_lv::setDir(const QString &s)
{
	cerr << "setDir called with argument: " << s << endl;
}*/

/*
void directory_lv::openFolder()
{
    autoopen_timer->stop();
    if ( dropItem && !dropItem->isOpen() ) {
	dropItem->setOpen( TRUE );
	dropItem->repaint();
    }
}
*/

file_dir_lv::file_dir_lv(QWidget *parent, const char *name = 0, bool sdo = FALSE)
	: directory_lv(parent, name, sdo)
{
	root = new directory_lv_item(this, "/");
}

void file_dir_lv::add_file(const QString &file, const QString &size)
{
	directory_lv::add_file(root, file, size);
}

void file_dir_lv::clear_files()
{
	delete root;
}

search_dir_lv::search_dir_lv(QWidget *parent = 0, const char *name = 0, bool sdo = FALSE)
	: directory_lv(parent, name, sdo)
{
}

void search_dir_lv::add_file(const QString &user, const QString &file, const QString &size, 
		const QString &slot)
{
	directory_lv_item *root = get_user(user);

	if (!root)
	{
		root = new directory_lv_item(this, user);
		users.push_back(root);
		root->setText(1, slot);
	}

	directory_lv::add_file(root, file, size);	
}

directory_lv_item *search_dir_lv::get_user(const QString &user)
{
	if (users.size() == 0) return 0;
	if (users.size() == 1)
	{
		if (users.front() == user)
			return users.front();
		else
			return 0;
	}
	
	list<directory_lv_item*>::iterator loc = ::find(users.begin(), users.end(), user);

	if (loc == users.end() && users.back() != user)
		return 0;
	else
		return *loc;
}

void search_dir_lv::clear_files()
{
	while (!users.empty())
	{
		directory_lv_item *d = users.front();
		users.pop_front();
		delete d;
	}
}
