/* This file is part of dc_qt.

    dc_qt is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    dc_qt is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dc_qt; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "file_transfer.h"
#include "util.h"

#include <qtimer.h>
#include <sys/stat.h>
#include <time.h>

file_transfer::file_transfer(int i, const QString &u, const QString &f, const QString &s)
	: user(u), file(f)
{
	done = false;
	
	total_size = s.toLong();
	bytes_transferred = orig_size = get_bytes();

//	QTimer *timer = new QTimer(this, "file_transfer timer");
//	connect(timer, SIGNAL(timeout()), SLOT(update_amt_transferred()));
//	timer->start(1000);

	time(&time_started);

	id = i;
}

off_t file_transfer::get_bytes()
{
	struct stat statbuf;

	stat(file.latin1(), &statbuf);

	return statbuf.st_size;
}

void file_transfer::update_amt_transferred()
{
	bytes_transferred = get_bytes();

	if (bytes_transferred == total_size)
		done = true;
}

float file_transfer::bps() const
{
	time_t cur_time;

	time(&cur_time);

	time_t time_elapsed = cur_time - time_started;

	if (!time_elapsed)
		return 0.0;

	return double(bytes_transferred - orig_size) / time_elapsed;
}
