#include "file_transfer_lv.h"
#include "dc_file.h"

#include <qtimer.h>
#include <qobject.h>
#include <qpopupmenu.h>
#include <algorithm>

const int CANCEL = 0;
const int FLIST = 1;

file_transfer_lv::file_transfer_lv(QWidget* parent, const char *name)
	: QListView(parent, name)
{
	timer = new QTimer(this);
	timer->start(1000);

	addColumn("Nick");
	addColumn("File");
	addColumn("Size");
	addColumn("Percent");
	addColumn("Speed");
	addColumn("ETA");

	menu = new QPopupMenu(this, "file_transfer_lv menu");
	menu->insertItem("Cancel", CANCEL);
	menu->insertItem("Get User File List", FLIST);

	connect (menu, SIGNAL(activated(int)), SLOT(menu_clicked(int)));
	connect (this, SIGNAL(rightButtonClicked(QListViewItem*, const QPoint&, int)),
			SLOT(show_menu(QListViewItem*, const QPoint&, int)));

	last_clicked = 0;
}

void file_transfer_lv::add_transfer(int id, const QString &u, const QString &f, const QString &s)
{
	file_transfer_lv_item *item = new file_transfer_lv_item(this, id, u, f, s);

	connect(timer, SIGNAL(timeout()), SLOT(update_all()));
	
	insertItem(item);
	
	transfers.push_back(item);
}

void file_transfer_lv::remove_transfer(int id)
{
	if (transfers.size() == 0)
		return;
	
	list<file_transfer_lv_item*>::iterator i = transfers.begin();

	do
	{
		if ((*i)->transfer_id() == id)
		{
			file_transfer_lv_item *temp = *i;
			transfers.erase(i);
			delete temp;
			break;
		}
		
		i++;
	} while (i != transfers.end());
}

void file_transfer_lv::remove_transfer(const QString &u, const QString &f)
{
	if (transfers.size() == 0)
		return;
	
	list<file_transfer_lv_item*>::iterator i = transfers.begin();

	file_transfer del(-1, u, f, "-1");

	do
	{
		if (**i == del)
		{
			file_transfer_lv_item *temp = *i;
			transfers.erase(i);
			delete temp;
			break;
		}
		
		i++;
	} while (i != transfers.end());
}

void file_transfer_lv::update_all()
{
	if (transfers.size() == 0)
		return;
	
	list<file_transfer_lv_item*>::iterator i = transfers.begin();

	do
	{
		(*i)->update_data();
		i++;
	} while (i != transfers.end());
}

void file_transfer_lv::menu_clicked(int i)
{
	if (!last_clicked)
		return;
	
	if (i == CANCEL)
		emit cancel_transfer (last_clicked->transfer_id());

	if (i == FLIST)
		emit get_flist(last_clicked->transfer_user());
}

void file_transfer_lv::show_menu(QListViewItem* i, const QPoint &p, int)
{
	menu->popup(p);
	last_clicked = (file_transfer_lv_item*)i;
}
