#include "preferences_dlg.h"
#include "dc_settings.h"

#include <qlayout.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qlistview.h>
#include <qfiledialog.h>
#include <qcombobox.h>

preferences_dlg::preferences_dlg(QWidget *p,
			     const char *name,
			     bool modal)
  : QDialog(p, name, modal)
{
	bool ok;        // used for conversion checking
  
	QGridLayout *grid = new QGridLayout(this,4,1);
  
	QHBoxLayout *bottom = new QHBoxLayout;
	grid->addLayout(bottom,3,0);
	QPushButton *okBtn = new QPushButton("&OK",this);
	QPushButton *cancelBtn = new QPushButton("&Cancel",this);
	bottom->addWidget(okBtn);
	bottom->addWidget(cancelBtn);
	connect(okBtn,SIGNAL(clicked()),this,SLOT(update_settings()));
	connect(okBtn,SIGNAL(clicked()),this,SLOT(close()));
	connect(cancelBtn,SIGNAL(clicked()),this,SLOT(close()));

	QGroupBox *userbox = new QGroupBox("User",this,"user box");
	grid->addWidget(userbox,0,0);

	QGroupBox *networkbox = new QGroupBox("Network",this,"network box");
	grid->addWidget(networkbox,1,0);

	QVGroupBox *dirbox = new QVGroupBox("Directories",this,"dir box");
	grid->addWidget(dirbox,2,0);

	// Name
	QGridLayout *userGrid = new QGridLayout(userbox,3,2,14);
	QLabel *editLbl = new QLabel("Name:", userbox);
	nameEdit = new QLineEdit(settings.get_setting(dc_settings::NICKNAME),
							userbox,"nameedit");
	// Description
	QLabel *descLbl = new QLabel("Description:",userbox);
	descEdit = new QLineEdit(settings.get_setting(dc_settings::DESCRIPTION),
							userbox,"descedit");

	userGrid->addWidget(editLbl,0,0);
	userGrid->addWidget(nameEdit,0,1);
	userGrid->addWidget(descLbl,1,0);
	userGrid->addWidget(descEdit,1,1);

	// Network section

	QGridLayout *netGrid = new QGridLayout(networkbox,6,2,14);
	
	// Port
	QLabel *portLbl = new QLabel("Port:",networkbox);
	int portnr = settings.get_setting(dc_settings::PORT).toInt(&ok);
	if(ok == false) cerr << "Syntax error in port number.\n";
	portSpin = new QSpinBox(0,65535,1,networkbox);
	portSpin->setValue(portnr);
	
	// Slots
	QLabel *slotsLbl = new QLabel("Slots:",networkbox);
	int slotnr = settings.get_setting(dc_settings::SLOTS).toInt(&ok);
	if(ok == false) cerr << "Syntax error in slots number.\n";
	slotsSpin = new QSpinBox(1,15,1,networkbox);
	slotsSpin->setValue(slotnr);

	// Max upload speed
	QLabel *speedLbl = new QLabel("Max upload speed (kb/s):",networkbox);
	int limit = settings.get_setting(dc_settings::LIMIT).toInt(&ok);
	if(ok==false) cerr << "Syntax error in upload limit.\n";
	speedSpin = new QSpinBox(0,99999,1,networkbox);
	speedSpin->setValue(limit);

	// Address
	QLabel *addrLbl = new QLabel("Forced IP (leave blank for default):",networkbox);
	addrEdit = new QLineEdit(settings.get_setting(dc_settings::ADDRESS),
			   networkbox,"Address");

	// Connection type, sorry for the ugly code.
	connectionCombo = new QComboBox(networkbox,"connectiontype");
	QStringList connTypes = settings.get_connection_types();
	int connTypeIndex = connTypes.findIndex(settings.get_setting(dc_settings::CONNECTION));
	connectionCombo->insertStringList( connTypes );
	connectionCombo->setCurrentItem(connTypeIndex);
  
	// passive
	passive = new QCheckBox("passive connection",networkbox);
	if(settings.get_setting(dc_settings::PASSIVE) == "true")
		passive->setChecked(true);

	netGrid->addWidget(portLbl,0,0);
	netGrid->addWidget(portSpin,0,1);
	netGrid->addWidget(slotsLbl,1,0);
	netGrid->addWidget(slotsSpin,1,1);
	netGrid->addWidget(speedLbl,2,0);
	netGrid->addWidget(speedSpin,2,1);
	netGrid->addWidget(addrLbl,3,0);
	netGrid->addWidget(addrEdit,3,1);
	netGrid->addWidget(connectionCombo,4,1);
	netGrid->addWidget(passive,5,1);

	// Folders section TODO DnD
	// shared
	shared = new QListView(dirbox,"sharedlist");
	shared->addColumn("Directory");
  
	for(int i=0;i < settings.num_shares();i++)
		QListViewItem *directory = new QListViewItem(shared,settings.get_share(i));

	QHBox *hbox1 = new QHBox(dirbox);

	QPushButton *add = new QPushButton("add",hbox1);
	QPushButton *remove = new QPushButton("remove",hbox1);

	connect(add,SIGNAL(clicked()),this,SLOT(addSharesDlg()));
	connect(remove,SIGNAL(clicked()),this,SLOT(removeSelectedShare()));

	// DL path
	QHBox *hbox2 = new QHBox(dirbox);
	QLabel *dlLbl = new QLabel("Download path:",hbox2);
	dlEdit = new QLineEdit(settings.get_setting(dc_settings::DLDIR),hbox2,"dlpath"); 

	QPushButton *dlBrowseBtn = new QPushButton("...",hbox2);
	connect(dlBrowseBtn,SIGNAL(clicked()),this,SLOT(openDirSelDlg()));
}

void preferences_dlg::openDirSelDlg()
{
	QString dir = QFileDialog::getExistingDirectory(settings.get_home_dir(), this, "dldir_dialog",
													"Select download directory");
	dlEdit->setText(dir);
}

void preferences_dlg::removeSelectedShare()
{
	shared->takeItem( shared->selectedItem() );
}

void preferences_dlg::addSharesDlg()
{
	QString dir = QFileDialog::getExistingDirectory(settings.get_home_dir(), this, "shdir_dialog",
													"Select a sharing directory");

	if(dir != QString::null)
		QListViewItem *item = new QListViewItem(shared,dir);
}

//  update_settings()
//   Typically called when someone presses OK.
//   writes all config values to the preferences object,
void preferences_dlg::update_settings()
{
	settings.set_setting( dc_settings::NICKNAME, nameEdit->text() );
	settings.set_setting( dc_settings::DESCRIPTION, descEdit->text() );
	settings.set_setting( dc_settings::ADDRESS, addrEdit->text() );
	settings.set_setting( dc_settings::SLOTS, slotsSpin->text() );
	settings.set_setting( dc_settings::PORT, portSpin->text() );
	settings.set_setting( dc_settings::CONNECTION, connectionCombo->currentText() );
	settings.set_setting( dc_settings::PASSIVE, passive->isChecked() ? "true" : "false");
	settings.set_setting( dc_settings::LIMIT, speedSpin->text() );
	settings.set_setting( dc_settings::DLDIR, dlEdit->text() );

	// Add shares
	settings.clear_shares();
	QListViewItem *item = shared->firstChild();
	while(item) 
	{
		settings.add_share(item->text(0));
		item = item->nextSibling();
	}

#ifdef _DEBUG
	settings.print();
#endif

	settings.save_to_file();
}
