/* This file is part of dc_qt.

    dc_qt is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    dc_qt is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dc_qt; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "user_lv_item.h"
#include "util.h"

#include <string.h>
#include <qpopupmenu.h>

const int CONNECTION = 0;
const int NAME = 1;
const int SHARED = 2;
const int DESCRIPTION = 3;

user_lv_item::user_lv_item(QListView *parent, const QString &n)
		: QListViewItem(parent, " ", n, " ", " ")
{
	shared_bytes = 0;
	
//	menu = new QPopupMenu(this, "user context menu");
//
//	menu->insertItem("&Get File List", this, SIGNAL(get_file_list()));
}

bool user_lv_item::operator==(const QString &o)
{
	return text(NAME) == o;
}

#ifdef _QT_3
// make sure file size is sorted by properly.  One of my pet peeves
// with dc_gui
int user_lv_item::compare(QListViewItem *i, int col, bool ascending) const
{
	if (col == NAME)
	{
		const char * s1 = text(NAME);
		const char * s2 = i->text(NAME);

		return strcasecmp(s1, s2);
	}

	if (col == SHARED)
	{
		double sb_this = user_lv_item::get_shared_bytes(text(SHARED));
		double sb_other = user_lv_item::get_shared_bytes(i->text(SHARED));

		if (sb_this > sb_other)
			return 1;
		if (sb_this < sb_other)
			return -1;

		return 0;
	}

	return key(col, ascending).compare(i->key(col, ascending));
}
#else
QString user_lv_item::key(int column, bool ascending) const
{
	if (column == SHARED)
	{
		QString s;
		s.sprintf("%lf", get_shared_bytes(text(SHARED)));
		return s;
	}

	return text(column);
}
#endif

void user_lv_item::connection(const QString &s)
{
	setText(CONNECTION, s);
}

// this should be extracted, because it's useful in other places, like 
// file_list_dialog
QString user_lv_item::get_shared_string(const QString &s)
{
	return bytes_to_units(s);
}

// same as above
double user_lv_item::get_shared_bytes(const QString &s)
{
	return units_to_bytes(s);
}

void user_lv_item::shared(const QString &s)
{
	setText(SHARED, get_shared_string(s));
}

void user_lv_item::description(const QString &s)
{
	setText(DESCRIPTION, s);
}

bool operator==(user_lv_item *i, const QString &o)
{
	return i->operator==(o);
}

bool operator!=(user_lv_item *i, const QString &o)
{
	return !i->operator==(o);
}
