#include "util.h"

#include <qstring.h>
#include <assert.h>
#include <iostream>
#include <stdio.h>

QString bytes_to_units(const QString &s)
{
	return bytes_to_units(s.toDouble());
}

QString bytes_to_units(double sb)
{
	QString t;

	if (sb > 1000000000000.0)
		t.sprintf("%.2lfTB", sb / 1000000000000.0);
	else if (sb > 1000000000.0)
		t.sprintf("%.2lfGB", sb / 1000000000.0);
	else if (sb > 1000000.0)
		t.sprintf("%.2lfMB", sb / 1000000.0);
	else if (sb > 1000.0)
		t.sprintf("%.2lfKB", sb / 1000.0);
	else
		t.sprintf("%.2lfBT", sb);
	
	return t;
}

double units_to_bytes(const QString &s)
{
	QString num(s.left(s.length() - 2));
	QString units(s.right(2));

	double ret = num.toDouble();

	if (units == "KB")
		ret *= 1000.0;
	else if (units == "MB")
		ret *= 1000000.0;
	else if (units == "GB")
		ret *= 1000000000.0;
	else if (units == "TB")
		ret *= 1000000000000.0;

	return ret;
}

QString sec_to_time(float seconds)
{
	if (seconds <= 0)
		return "0:0:0";

	// we can be passed in seconds=inf
	if (seconds >= 3600 * 24 * 2)
		return "> 2 days";
	
	int hours = 0, min = 0;

	while (seconds >= 3600)
	{ hours++; seconds -= 3600; }
	while (seconds >= 60)
	{ min++; seconds -= 60; }

	QString result;
	result.sprintf("%d:%02d:%02d", hours, min, int(seconds + 0.5));

	return result;
}

#ifdef _VERBOSE
void log(const QString &l)
{
	cout << l;
	
	if (l[l.length() - 1] != '\n')
		cout << endl;
}

void log(int i, const QString &name = QString::null)
{
	cout << name << ": " << i << endl;
}

void log(long l, const QString &name = QString::null)
{
	cout << name << ": " << l << endl;
}

void log(double d, const QString &name = QString::null)
{
	cout << name << ": " << d << endl;
}

void log(void* v, const QString &name = QString::null)
{
	printf("%s: %x", name.latin1(), v);
}
#endif

void errlog(const QString &l)
{
	cerr << l;

	if (l[l.length() - 1] != '\n')
		cerr << endl;
}
